<?php
/**
 * Processes credit card payments, calls payadvantage.
 */

include_once(plugin_dir_path(__FILE__) . '/payadvantage-json-converter.php');
include_once(plugin_dir_path(__FILE__) . '/payadvantage-validator.php');
include_once(plugin_dir_path(__FILE__) . '../shared/payadvantage-result-processor.php');

class Pay_Advantage_Credit_Card_Payment
{

  protected $pay_advantage_url = '';
  protected $access_token = '';

  /**
   * This method orchestrates the credit card payments, 
   */
  public function process_credit_card_payment($billing_data, $amount, $description)
  {
    $json_converter = new Pay_Advantage_Json_Converter();
    $auth_provider = new Pay_Advantage_Auth();
    $this->pay_advantage_url = get_option('pay_advantage_url');
    $token = $auth_provider->get_auth_token(null, null);

    if (isset($token['ErrorMessages']) > 0)
      return $token;

    $this->access_token = $token['access_token'];

    $customer_code = $this->get_customer_code($billing_data);
    if (isset($customer_code['ErrorMessages'])) {
      return $customer_code;
    }

    $json_credit_card_token_request = $json_converter->create_credit_card_token_json($customer_code);
    $credit_card_token = $this->create_token_for_credit_card($json_credit_card_token_request);

    if (isset($credit_card_token['ErrorMessages'])) {
      return $credit_card_token;
    }

    $json_charge = $json_converter->create_payment_credit_card_charge_json($amount, $description);
    $payment_result = $this->charge_credit_card($credit_card_token['Code'], $json_charge);
    return $payment_result;
  }

  /**
   * Gets the customer code, creates the customer if it has to or returns an error.
   */
  private function get_customer_code($billing_data)
  {
    $json_converter = new Pay_Advantage_Json_Converter();
    $query_customer_result = $this->customer_query_api_call(
      $billing_data['email'],
      $billing_data['first_name'],
      $billing_data['last_name']
    );
    if (isset($query_customer_result['ErrorMessages'])) {
      return $query_customer_result;
    }

    $existing_customer = $query_customer_result['Records'];

    if ($existing_customer == array()) {
      $json_customer = $json_converter->create_customer_json($billing_data);
      $customer = $this->create_customer_for_credit_card_payment($json_customer);

      if (isset($customer['ErrorMessages'])) {
        return $customer;
      }

      return $customer_code = $customer['Code'];
    } else {
      return $customer_code = $existing_customer[0]['Code'];
    }
  }

  /**
   * Queries the customer to see if they're existing.
   */
  private function customer_query_api_call($email, $first_name, $last_name)
  {
    $result_processor = new Pay_Advantage_Result_Processor();
    $url = "$this->pay_advantage_url/customers?email=$email&firstname=$first_name&lastname=$last_name";
    $headers = array(
      'headers' => array(
        'Content-Type' => 'application/json',
        'Authorization' => "Bearer $this->access_token"
      ),
      'timeout' => 30
    );

    $response = wp_remote_get($url, $headers);
    return $result_processor->process_result($response, 'Query', 'creditcard');
  }

  /**
   * Creates customer
   */
  private function  create_customer_for_credit_card_payment($json_customer)
  {
    $result_processor = new Pay_Advantage_Result_Processor();
    $json_converter = new Pay_Advantage_Json_Converter();
    $url = "$this->pay_advantage_url/customers";
    $data = array(
      'headers' => array(
        'Content-Type' => 'application/json',
        'Expect' => '',
        'Authorization' => "Bearer $this->access_token"
      ),
      'body' => $json_customer,
      'timeout' => 30
    );

    $response = wp_remote_post($url, $data);

    return $result_processor->process_result($response, 'Query', 'creditcard');
  }

  /**
   * Hits api for Credit Card token
   */
  private function create_token_for_credit_card($json_card_info)
  {
    $result_processor = new Pay_Advantage_Result_Processor();
    $url = "$this->pay_advantage_url/credit_cards";
    $data = array(
      'headers' => array(
        'Content-Type' => 'application/json',
        'Expect' => '',
        'Authorization' => "Bearer $this->access_token"
      ),
      'body' => $json_card_info,
      'timeout' => 30
    );

    $response = wp_remote_post($url, $data);

    return $result_processor->process_result($response, 'token', 'creditcard');
  }

  /**
   * Sends the api call to charge the consumer.
   */
  private function charge_credit_card($credit_card_code, $charges_json)
  {
    $result_processor = new Pay_Advantage_Result_Processor();
    $url = "$this->pay_advantage_url/credit_cards/$credit_card_code/charges";
    $data = array(
      'headers' => array(
        'Content-Type' => 'application/json',
        'Expect' => '',
        'Authorization' => "Bearer $this->access_token"
      ),
      'body' => $charges_json,
      'timeout' => 30
    );

    $response = wp_remote_post($url, $data);

    return $result_processor->process_result($response, 'charge', 'creditcard');
  }
}
