<?php
/**
 * This file is for the WooCommerce payment gateway. It has its own options for merchant id and password. Uses the credit card class to process payment.
 */

include_once( plugin_dir_path( __FILE__ ) . 'payadvantage-json-converter.php' );
include_once( plugin_dir_path( __FILE__ ) . '../shared/payadvantage-api-error-handler.php' );
include_once( plugin_dir_path( __FILE__ ) . '../shared/payadvantage-result-processor.php' );
include_once( plugin_dir_path( __FILE__ ) . 'payadvantage-validator.php' );
include_once( plugin_dir_path( __FILE__ ) . 'payadvantage-creditcard.php' );
include_once( plugin_dir_path( __FILE__ ) . 'html/woocommerce-payment-fields-html.php' );

add_action('plugins_loaded', 'pay_advantage_gateway_handler');

function pay_advantage_gateway_handler() {
    if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
        return;
    }

    class Pay_Advantage_Gateway extends WC_Payment_Gateway {
        protected $api_username = '';
        protected $api_password = '';
        protected $order = null;
       
        /**
         * This function sets up variables for WooCommerce, it tells Woo the inner workings of the class   
         */         
        public function __construct() {
            $this->id = 'pay_advantage_gateway';
            $this->title = 'Credit Card (Pay Advantage)';
            $this->icon = null;
            $this->has_fields = true;
            $this->method_title = 'Pay Advantage';
            $this->method_description = 'A payment solution provider Pay Advantage.';
            $this->init_form_fields();
            $this->init_settings();
            
            $this->api_username = $this->settings['payadvantagewoogatewayusername'];
            $this->api_password = $this->settings['payadvantagewoogatewaypassword'];
           
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
        }

   /**
    *  Not currently used but mandatory, could be used later for a settings page inside WooCommerce    
    */
    function init_form_fields() {
        $this->form_fields = array(
            'payadvantagewoogatewayusername' => array(
                'title' => __( 'API Username', 'woocommerce' ),
                'type' => 'text',
                'description' => __( 'Your merchant username is required.' ),
                'default' => __( '', 'woocommerce' ),
                'desc_tip'    => true,
            ),
            'payadvantagewoogatewaypassword' => array(
                'title' => __( 'API Password', 'woocommerce' ),
                'type' => 'password',
                'description' => __( 'Your merchant password is required.' ),
                'default' => __( '', 'woocommerce' ),
                'desc_tip'    => true,
            )
        );
    }

    /**
     * This method is connected to the has_fields method. It contains html for collecting data in the checkout window
     * like credit card info.
     */
    function payment_fields() {
        echo pay_advantage_woocommerce_payment_fields();
    }

    /**
     * This is validating fields in the payment gateway. 
     */
    public function validate_fields() {
        global $woocommerce;
        $validator = new Pay_Advantage_Validator();
        
        // TODO: Sanitize first
        
        if ( ! $validator->credit_card_number( $_POST['payadvantagecardnumber'] ) ) {
            wc_add_notice( __( 'The credit card number you entered is invalid.', 'woocommerce' ), 'error' );
        }
        if ( ! $validator->card_holders_name( $_POST['payadvantagecardholdersname'] ) ) {
            wc_add_notice( __( 'The card holders name you entered is invalid.', 'woocommerce' ), 'error' );
        }
        if ( ! $validator->credit_card_ccv( $_POST['payadvantagecreditcardcode'] ) ) {
            wc_add_notice( __( 'The credit card ccv number you entered is invalid.', 'woocommerce' ), 'error' );
        }
        if ( ! $validator->credit_card_expiry( $_POST['payadvantageexpirymonth'], $_POST['payadvantageexpiryyear'] ) ) {
            wc_add_notice( __( 'The credit card expiry is invalid.', 'woocommerce' ), 'error' );
        }
    }

    /**
    * This is the meat and bones of the WooCommerce integration. It picks up information from the place order button and processes it.
    */
    function process_payment( $order_id ) {
        global $woocommerce;
        $order = new WC_Order( $order_id );
        $this->$order = $order;
        $order_data = $order->data;
            
        if ( $order_data == null ) {
            return array(
                'result' => 'error'
            );
        }

        $billing = $order_data['billing'];

        if ( $billing == null ) {
            return array(
                'result' => 'error'
            );
        }

        $is_payment_successful = $this->process_credit_card_payment( $billing, $order_data['total'], "Order: " . $order_id );
        
        // Return thankyou redirect
        if ( $is_payment_successful ) {
            $this->payment_successful( $order, $woocommerce );

            return array(
                'result' => 'success',
                'redirect' => $this->get_return_url( $order )
            );
        } else {
            return array(
                'result' => 'failed',
            );
        }
    }

    /**
     * Orchestrates the credit card payment
     */
    private function process_credit_card_payment( $billing_data, $amount, $description ) {
        $credit_card_api = new Pay_Advantage_Credit_Card_Payment();

        $result = $credit_card_api->process_credit_card_payment( $billing_data, $amount, $description );

        if ( isset( $result['ErrorMessages'] ) ) {
            $error = $this->mark_as_failed_payment( $result['ErrorMessages'] );

            wc_add_notice( __( '(Transaction Error) There was an error processing your payment request. ' . $error, 'woocommerce' ),'error' );
            return false;
        }

        if ( isset( $result['ChargeStatus'] ) ) {
            if ( $result['ChargeStatus'] == 'approved' ) {
                return true;
            }
        }
        return false;
    }

    /**
     * If payment is successful this is called to process the order
     */
    private function payment_successful( $order, $woocommerce ) {
        // Updates status to complete.
        $order->update_status( 'completed', __( 'Payed by credit card', 'woocommerce' ) );
        // Reduce stock levels.
        $order->reduce_order_stock();
        // Remove cart.
        $woocommerce->cart->empty_cart();
    }

    /**
     * Order failed
     */
    private function mark_as_failed_payment( $error_messages ) {
        $error = '';
        foreach ( $error_messages as $error_message ) {
            $error = $error . '<br>'. $error_message;            
        }
        return $error;
        }
    }
}

/**
 * This is required as per WooCommerce documentation.
 * It lists methods into an array for use by parent class.
 */
function Pay_Advantage_Gateway( $methods ) {
    $methods[] = 'Pay_Advantage_Gateway'; 
    return $methods;
}
add_filter( 'woocommerce_payment_gateways', 'Pay_Advantage_Gateway' );
?>